/*
 * Form handler for the titrax main form
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: mainForm.c,v 1.5 1999/06/23 13:08:15 ga58 Exp $
 *
 * $Log: mainForm.c,v $
 * Revision 1.5  1999/06/23 13:08:15  ga58
 * Version 3.34
 *
 * Revision 1.4  1999/06/03 12:04:35  ga58
 *  - Version 3.32
 *
 * Revision 1.3  1999/02/28 16:36:40  ga58
 *  - Version 3.31
 *
 * Revision 1.2  1999/02/27 17:39:03  ga58
 *  - Version 3.30
 *
 * Revision 1.1  1999/02/21 13:41:56  ga58
 *  - Version 3.29
 *
 * Revision 1.7  1999/02/14 17:58:44  ga58
 *  - Version: 3.28
 *
 * Revision 1.6  1999/01/31 19:05:07  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 * Revision 1.5  1999/01/30 19:08:15  ga58
 *  - Version 3.24. Dynamically resizable number of project slots.
 *
 * Revision 1.4  1999/01/24 20:22:57  ga58
 *  - Beta version 3.23
 *
 * Revision 1.3  1999/01/23 16:02:07  ga58
 *  - checkin before removing all row-constructs
 *
 * Revision 1.2  1999/01/17 20:02:18  ga58
 *  - nearly complete rewrite of titrax and all its modules
 *
 */

#include "titraxAll.h"
#include "titraxID.h"
#include "titraxConst.h"
#include "titraxPref.h"
#include "titraxUtil.h"
#include "slotName.h"
#include "slotNameList.h"
#include "project.h"
#include "titraxCommands.h"
#include "mainForm.h"
#include "TxtGlue.h"

static Boolean inList = false;

#define minDnD 6

static void
MainFormCheckScroll()
{
  SclSetScrollBar(GetObjectPtr(mainFormScrollBar), Prefs()->firstVisibleSlot, 
						 0, Prefs()->maxSlot - VISSLOT, VISSLOT);
}

static void
ScrollTo(Int16 slot)
{
  if (slot == Prefs()->firstVisibleSlot)
	 return;

  if (slot < 0)
	 slot = 0;

  if (slot > (Prefs()->maxSlot - VISSLOT))
	 slot = Prefs()->maxSlot - VISSLOT;

  Prefs()->firstVisibleSlot = slot;

  UpdateAllSlots();
  MainFormCheckScroll();
}

static void
ScrollUp(Int16 amount)
{
  ScrollTo(Prefs()->firstVisibleSlot - amount);
}

static void
ScrollDown(Int16 amount)
{
  ScrollTo(Prefs()->firstVisibleSlot + amount);
}

void
ScrollCenterActive()
{
  if (ActiveSlot() == noSlotActive)
	 return;

  ScrollTo(ActiveSlot() - ((VISSLOT - 1) / 2));
}

static Int16
YPositionToSlot(Int16 y)
{
  Int16 tableRow;
  RectangleType rect;
  TablePtr tableP = GetObjectPtr(mainFormTable);

  for(tableRow = 0; tableRow < VISSLOT; tableRow++) {
	 TblGetItemBounds(tableP, tableRow, 0, &rect);
	 if (RctPtInRectangle(rect.topLeft.x + 1, y, &rect))
		break;
  }

  return tableRow + Prefs()->firstVisibleSlot;
}

static Int16
XPositionToColumn(Int16 x)
{
  Int16 column;
  RectangleType rect;
  TablePtr tableP = GetObjectPtr(mainFormTable);

  for(column = 0; column < MAXCOL; column++) {
	 TblGetItemBounds(tableP, 0, column, &rect);
	 if (x < (rect.topLeft.x + rect.extent.x))
		break;
  }

  return column;
}

void
DrawSlot(MemPtr tableP, Int16 row, Int16 column, RectanglePtr bounds)
{
  const Int16 strLen = 4 * 40;
  Int16 slot;
  Char text[strLen + 1];
  FontID oldFont;

  slot = row + Prefs()->firstVisibleSlot;

  TblUnhighlightSelection(tableP);
  WinEraseRectangle(bounds, 0);

  switch (column) {
  case 0: {
	 oldFont = FntSetFont(stdFont);
	 if (!SlotIsEmpty(slot)) {
		GetSlotName(slot, text, strLen);
		TrimToLength(text, stdFont, BUTTONW - 2);
	 }
	 else
		StrNCopy(text, " \205", strLen);
	 break;
  }

  case 1: {
	 oldFont = FntSetFont(stdFont);
	 if (!SlotIsEmpty(slot)) {
		TimeToString(TotalElapsed(slot), text, strLen);
		ExpandLeftToLength(text, strLen, TIMEW-1);
	 }
	 else
		StrNCopy(text, "", strLen);
	 break;
  }

  case 2: {
	 oldFont = FntSetFont(symbolFont);
	 if (!SlotIsEmpty(slot)) {
		if (NoteIsEmpty(slot))
		  StrNCopy(text, "\013", strLen);
		else
		  StrNCopy(text, "\014", strLen);
	 }
	 else
		StrNCopy(text, "", strLen);
	 break;
  }

  default:
	 return;
  }
  
  WinDrawChars(text, StrLen(text), bounds->topLeft.x, bounds->topLeft.y);
  FntSetFont(oldFont);

  if ((ActiveSlot() == slot) && ((column == 0) || (column == 1)))
	 WinInvertRectangle(bounds, 0);
  
}

static void
MainFormInit(void)
{
  Int16 tableRow;
  Int16 column;
  FormPtr frm = FrmGetActiveForm();
  TablePtr tableP = GetObjectPtr(mainFormTable);
  RectangleType rect1 = {{0, 139}, {160, 2}};

  for(tableRow = 0; tableRow < VISSLOT; tableRow++) {
	 TblSetRowUsable(tableP, tableRow, true);

	 for(column = 0; column < MAXCOL; column++)
		TblSetItemStyle(tableP, tableRow, column, customTableItem);
  }
  
  for(column = 0; column < MAXCOL; column++) {
	 TblSetCustomDrawProcedure(tableP, column, DrawSlot);
	 TblSetColumnUsable(tableP, column, true);
  }

  TblSetColumnSpacing(tableP, 0, 0);
  TblSetColumnSpacing(tableP, 1, SPACE);
  TblSetColumnSpacing(tableP, 2, 0);

  SlotNameListUpdate();
  UpdateProjectListButton();

  CtlSetValue(GetObjectPtr(mainFormNewUI), Prefs()->newUI);

  FrmDrawForm(frm);

  MainFormCheckScroll();
  UpdateSumTimeButton();

  WinDrawRectangle(&rect1, 0);
}

Boolean
MainFormHandleEvent(EventPtr event)
{
  Boolean handled = false;
  TablePtr tableP = GetObjectPtr(mainFormTable);
  RectangleType rect, listRect;
  UInt16 listObjID;

  switch (event->eType) {
  case ctlSelectEvent: {
	 if (event->data.ctlSelect.controlID == mainFormCommand) {
		ListPtr listP;
		inList = true;

		if (ActiveSlot() != noSlotActive) {
		  listP = GetObjectPtr(mainFormCommandList1);
		  listObjID = mainFormCommandList1;
		}
		else {
		  listP = GetObjectPtr(mainFormCommandList2);
		  listObjID = mainFormCommandList2;
		}

		LstSetSelection(listP, noListSelection);
		LstSetHeight(listP, LstGetNumberOfItems(listP));
	
		FrmGetObjectBounds(FrmGetActiveForm(), FrmGetObjectIndex(FrmGetActiveForm(), listObjID), &listRect);
		LstSetPosition(listP, 3, 155 - (listRect.extent.y));

		DoCommand(LstPopupList(listP), ActiveSlot());
		
		inList = false;
		handled = true;
	 }
		
	 if (event->data.ctlSelect.controlID == mainFormProjectPopup)
		handled = (SlotNameListNumOfEntries() == 0);

	 if (event->data.ctlSelect.controlID == mainFormNewUI) {
		Prefs()->newUI = !Prefs()->newUI;
		CtlSetValue(GetObjectPtr(mainFormNewUI), Prefs()->newUI);
		handled = true;
	 }
	 break;
  }

  case penDownEvent: {
	 Int16 xPos, yPos, xp, yp;
	 Int16 startSlot, startSlotOld;
	 Int16 startSlotY;
	 Int16 startRow, startRowOld, currentRow;
	 Boolean penDown;

	 if (inList) {
		handled = false;
		break;
	 }

	 EvtGetPen(&xp, &yp, &penDown);

	 TblGetBounds(tableP, &rect);
	 if (RctPtInRectangle(xp, yp, &rect)) {
		startSlotY = yp;
		startSlot = YPositionToSlot(yp);
		startSlotOld = startSlot;
		startRow = startSlot - Prefs()->firstVisibleSlot;
		startRowOld = startRow;

		// begin new code
		while (penDown) {
		  xPos = xp; 
		  yPos = yp;
		  if (!RctPtInRectangle(xPos, yPos, &rect)) {
			 RectangleType rowRect;
			 if (yPos < rect.topLeft.y) {
				ScrollUp(1);
				TblGetItemBounds(tableP, 0, 0, &rowRect);
				WinInvertRectangle(&rowRect, 0);
				TblGetItemBounds(tableP, 0, 1, &rowRect);
				WinInvertRectangle(&rowRect, 0);
			 }
			 if (yPos > rect.topLeft.y + rect.extent.y) {
				ScrollDown(1);
				TblGetItemBounds(tableP, VISSLOT-1, 0, &rowRect);
				WinInvertRectangle(&rowRect, 0);
				TblGetItemBounds(tableP, VISSLOT-1, 1, &rowRect);
				WinInvertRectangle(&rowRect, 0);
			 }
		  }
		  else {
			 Int16 currentSlot = YPositionToSlot(yPos);
			 currentRow = currentSlot - Prefs()->firstVisibleSlot;
			 if (currentRow != startRowOld) {
				RectangleType rowRect;
				TblGetItemBounds(tableP, startRowOld, 0, &rowRect);
				WinInvertRectangle(&rowRect, 0);
				TblGetItemBounds(tableP, startRowOld, 1, &rowRect);
				WinInvertRectangle(&rowRect, 0);
				TblGetItemBounds(tableP, currentRow, 0, &rowRect);
				WinInvertRectangle(&rowRect, 0);
				TblGetItemBounds(tableP, currentRow, 1, &rowRect);
				WinInvertRectangle(&rowRect, 0);
				startSlotOld = currentSlot;
				startRowOld = currentRow;
			 }
		  }
		  EvtGetPen(&xp, &yp, &penDown);
		};
		
		if (RctPtInRectangle(xPos, yPos, &rect)) {
		  Int16 endSlot = YPositionToSlot(yPos);
		  if (startSlot != endSlot) {
			 if (abs(startSlotY - yPos) >= minDnD) {
				SwapSlots(startSlot, endSlot);
				SlotNameListUpdate();
				UpdateProjectListButton();
			 }
			 UpdateAllSlots();
		  }
		  else {
			 Int16 column = XPositionToColumn(xPos);
			 Int16 slot = endSlot;
			 
			 if (Prefs()->newUI) {
				if (column < 2) {
				  Int16 retCode;
				  ListPtr listP;
				  
				  inList = true;
				  
				  if (SlotIsEmpty(slot))
					 listP = GetObjectPtr(mainFormProjList3);
				  else {
					 if (slot != ActiveSlot())
						listP = GetObjectPtr(mainFormProjList1);
					 else
						listP = GetObjectPtr(mainFormProjList2);
				  }
				  
				  LstSetHeight(listP, LstGetNumberOfItems(listP));
				  LstSetSelection(listP, noListSelection);
				  LstSetPosition(listP, xPos - 4, yPos - 4);
				  
				  retCode = LstPopupList(listP);
				  if (retCode != noListSelection)
					 DoCommand(retCode + startStop, slot);
				  
				  LstSetSelection(listP, noListSelection);
				  inList = false;
				}
				else { // column == 2
				  if (!SlotIsEmpty(slot))
					 DoCommand(editNote, slot);
				  else
					 UpdateSlot(slot);
				}
			 }
			 else { // no newUI
				if (column < 2) {
				  if (ActiveSlot() == slot)
					 StopAndUpdate(slot);
				  else {
					 if (SlotIsEmpty(slot))
						DoCommand(changeName, slot); // Change Name
					 if (!SlotIsEmpty(slot))
						StartAndUpdate(slot);
				  }
				}
				else { // column == 2
				  if (!SlotIsEmpty(slot))
					 DoCommand(editNote, slot);
				  else
					 UpdateSlot(slot);
				}
			 }
		  }
		}
		
		// end new code
		handled = true;
	 }
	 else {
		handled = false;
	 }
	 break;
  }

  case popSelectEvent:
	 if (event->data.popSelect.listID == mainFormProjectList) {
		UInt16 selection = event->data.popSelect.selection;
	 
		if (selection != noListSelection) {
		  Int16 slotNo = SlotNameListNumToSlot(selection);
		  if (ActiveSlot() == slotNo)
			 StopAndUpdate(slotNo);
		  else
			 StartAndUpdate(slotNo);

		  ScrollCenterActive();
		}
		handled = true; // we redraw the popuptrigger
	 }
	 break;

  case frmOpenEvent:
	 MainFormInit();
	 Prefs()->actualForm = mainForm;
	 handled = true;
	 break;

  case frmCloseEvent:
	 inList = false;
	 handled = false;
	 break;

  case nilEvent:
	 if (FrmGetActiveFormID() == mainForm) {
		UpdateActiveSlot();
		UpdateSumTimeButton();
	 }
	 handled = false;
	 break;

  case sclRepeatEvent:
	 ScrollTo(event->data.sclRepeat.newValue);
	 handled = false;
	 break;

  case keyDownEvent:
	 if (event->data.keyDown.chr == pageUpChr) {
		ScrollUp(VISSLOT);
		handled = true;
	 }

	 if (event->data.keyDown.chr == pageDownChr) {
		ScrollDown(VISSLOT);
		handled = true;
	 }

	 /* Inspired from tclogger, Thanx a lot!!! */
	 if (TxtCharIsHardKey(event->data.keyDown.modifiers, event->data.keyDown.chr)) {
		 if (!(event->data.keyDown.modifiers & poweredOnKeyMask)) {
			 UInt16 cardNo;
			 LocalID dbID;
			 DmSearchStateType searchState;
			 Err err;
			 UInt32 command = 'none';

			 // find out, on which hard-key Titrax is installed
			 if (PrefGetPreference(prefHard1CharAppCreator) == titraxAppType)
				 command = sysFileCDatebook;
			 if (PrefGetPreference(prefHard2CharAppCreator) == titraxAppType)
				 command = sysFileCAddress;
			 if (PrefGetPreference(prefHard3CharAppCreator) == titraxAppType)
				 command = sysFileCToDo;
			 if (PrefGetPreference(prefHard4CharAppCreator) == titraxAppType)
				 command = sysFileCMemo;

			 if (command != 'none') {
			 	DmGetNextDatabaseByTypeCreator(true, &searchState, sysFileTApplication,
						 command, true, &cardNo, &dbID);
			 	ErrNonFatalDisplayIf(!dbID, "Could not find app");
			 	if (dbID) {
				 	err = SysUIAppSwitch(cardNo, dbID, sysAppLaunchCmdNormalLaunch, NULL);
				 	ErrNonFatalDisplayIf(err, "Could not launch app");
			 	}
			 }
		 }
		 handled = true;
	 }
	 break;

  case frmTitleSelectEvent:
	 if (Prefs()->menuHack) {
		MenuBarPtr menuP = MenuInit(mainFormMenu);

		MenuSetActiveMenu(menuP);
		MenuDrawMenu(menuP);
		handled = true;
	 }
	 else
		handled = false;
	 break;

  case menuEvent:
	 MenuEraseStatus(0);
	 if (event->data.menu.itemID == mainFormMenuOption) {
		FrmGotoForm(optionForm);
		handled = true;
	 }
	 if (event->data.menu.itemID == mainFormMenuAbout) {
		FormPtr frm1 = FrmInitForm(aboutForm);
		FormPtr frm2 = FrmInitForm(aboutForm2);

		FrmDoDialog(frm1);
		FrmDoDialog(frm2);
		FrmDeleteForm(frm2);
		FrmDeleteForm(frm1);

		handled = true;
	 }
	 break;

  default:
	 handled = false;
  }

  return handled;
}

