/*
 * slotName.c
 *
 * Titrax database subroutines for project names
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: slotName.c,v 1.2 1999/02/28 16:36:48 ga58 Exp $
 *
 * $Log: slotName.c,v $
 * Revision 1.2  1999/02/28 16:36:48  ga58
 *  - Version 3.31
 *
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.3  1999/01/31 19:05:18  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 * Revision 1.2  1999/01/30 19:08:23  ga58
 *  - Version 3.24. Dynamically resizable number of project slots.
 *
 * Revision 1.1  1999/01/24 20:23:05  ga58
 *  - Beta version 3.23
 *
 */

#include "titraxAll.h"
#include "titraxConst.h"
#include "titraxPref.h"
#include "slotDBUtil.h"
#include "slotName.h"

static DmOpenRef TitraxDB;

#define titraxDBType 'Name'
#define titraxDBName "TitraxNameDB"
#define titraxDBVersion 0

MemHandle slotNameHandle;
Char** slotName;

static MemHandle slotHandHandle;
static MemHandle* slotHand;


/*
 * Database operations
 * 
 * These routines do NOT perform any error-checking
 */
void
OpenSlotNameDatabase(void)
{
  slotHandHandle = MemHandleNew(sizeof(MemHandle[Prefs()->maxSlot]));
  slotHand = MemHandleLock(slotHandHandle);

  slotNameHandle = MemHandleNew(sizeof(MemPtr[Prefs()->maxSlot]));
  slotName = MemHandleLock(slotNameHandle);

  OpenDatabase(&TitraxDB, titraxDBName, titraxDBType, titraxAppType, slotHand, (MemPtr*)slotName, 1, titraxDBVersion);
}

void
CloseSlotNameDatabase(void)
{
  CloseDatabase(TitraxDB, slotHand);

  MemHandleUnlock(slotNameHandle);
  MemHandleFree(slotNameHandle);

  MemHandleUnlock(slotHandHandle);
  MemHandleFree(slotHandHandle);
}

void
GetSlotName(Int16 slot, Char* charP, Int16 maxLen)
{
  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "GetSlotName");
  StrNCopy(charP, slotName[slot], maxLen);
}

Char*
SlotName(Int16 slot)
{
  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "SlotName");
  return slotName[slot];
}

void
SetSlotName(Int16 slot, Char* charP)
{
  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "SetSlotName");

  MemHandleUnlock(slotHand[slot]);
  DmReleaseRecord(TitraxDB, slot, true);

  slotHand[slot] = DmResizeRecord(TitraxDB, slot, StrLen(charP) + 1);
  slotName[slot] = MemHandleLock(slotHand[slot]);

  DmSet(slotName[slot], 0, StrLen(charP) + 1, 0);

  DmStrCopy(slotName[slot], 0, charP);
}

void
SwapSlotName(Int16 slot1, Int16 slot2)
{
  Char* tmpP;
  MemHandle tmpH;

  tmpH = MemHandleNew(StrLen(slotName[slot1]) + 1);
  tmpP = MemHandleLock(tmpH);
  StrCopy(tmpP, slotName[slot1]);

  SetSlotName(slot1, SlotName(slot2));
  SetSlotName(slot2, tmpP);

  MemHandleUnlock(tmpH);
  MemHandleFree(tmpH);
}
