/*
 * slotNote.c
 *
 * Titrax database subroutines for project notes
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: slotNote.c,v 1.2 1999/02/28 16:36:51 ga58 Exp $
 *
 * $Log: slotNote.c,v $
 * Revision 1.2  1999/02/28 16:36:51  ga58
 *  - Version 3.31
 *
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.3  1999/01/31 19:05:19  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 * Revision 1.2  1999/01/30 19:08:27  ga58
 *  - Version 3.24. Dynamically resizable number of project slots.
 *
 * Revision 1.1  1999/01/24 20:23:05  ga58
 *  - Beta version 3.23
 *
 */

#include "titraxAll.h"
#include "titraxConst.h"
#include "titraxPref.h"
#include "slotDBUtil.h"
#include "slotNote.h"

static DmOpenRef TitraxDB;

#define titraxDBType 'Note'
#define titraxDBName "TitraxNoteDB"
#define titraxDBVersion 0

MemHandle slotNoteHandle;
Char** slotNote;

static MemHandle slotHandHandle;
static MemHandle* slotHand;


/*
 * Database operations
 * 
 * These routines do NOT perform any error-checking
 */
void
OpenSlotNoteDatabase(void)
{
  slotHandHandle = MemHandleNew(sizeof(MemHandle[Prefs()->maxSlot]));
  slotHand = MemHandleLock(slotHandHandle);

  slotNoteHandle = MemHandleNew(sizeof(MemPtr[Prefs()->maxSlot]));
  slotNote = MemHandleLock(slotNoteHandle);

  OpenDatabase(&TitraxDB, titraxDBName, titraxDBType, titraxAppType, slotHand, (MemPtr*)slotNote, 1, titraxDBVersion);
}

void
CloseSlotNoteDatabase(void)
{
  CloseDatabase(TitraxDB, slotHand);

  MemHandleUnlock(slotNoteHandle);
  MemHandleFree(slotNoteHandle);

  MemHandleUnlock(slotHandHandle);
  MemHandleFree(slotHandHandle);
}

void
GetSlotNote(Int16 slot, Char* charP, UInt16 maxLen)
{
  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "GetSlotNote");
  StrNCopy(charP, slotNote[slot], maxLen);
}

Char*
SlotNote(Int16 slot)
{
  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "SlotNote");
  return slotNote[slot];
}

void
SetSlotNote(Int16 slot, Char* charP)
{
  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "SetSlotNote");

  MemHandleUnlock(slotHand[slot]);
  DmReleaseRecord(TitraxDB, slot, true);

  slotHand[slot] = DmResizeRecord(TitraxDB, slot, StrLen(charP) + 1);
  slotNote[slot] = MemHandleLock(slotHand[slot]);

  DmSet(slotNote[slot], 0, StrLen(charP) + 1, 0);

  DmStrCopy(slotNote[slot], 0, charP);
}

void
SwapSlotNote(Int16 slot1, Int16 slot2)
{
  Char* tmpP;
  MemHandle tmpH;

  tmpH = MemHandleNew(StrLen(slotNote[slot1]) + 1);
  tmpP = MemHandleLock(tmpH);
  StrCopy(tmpP, slotNote[slot1]);

  SetSlotNote(slot1, SlotNote(slot2));
  SetSlotNote(slot2, tmpP);

  MemHandleUnlock(tmpH);
  MemHandleFree(tmpH);
}

MemHandle
SlotNoteHandleGet(Int16 slot)
{
  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "SlotNoteHandleGet");

  MemHandleUnlock(slotHand[slot]);

  return (MemHandle)slotHand[slot];
}

void
SlotNoteHandleRelease(Int16 slot)
{
  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "SlotNoteHandleRelease");

  DmReleaseRecord(TitraxDB, slot, true);
  
  slotHand[slot] = DmGetRecord(TitraxDB, slot);
  slotNote[slot] = MemHandleLock(slotHand[slot]);
}
